package org.vraptor.plugin.hibernate;

import org.apache.log4j.Logger;
import org.vraptor.component.ComponentManager;
import org.vraptor.component.ComponentType;
import org.vraptor.component.LogicNotFoundException;
import org.vraptor.plugin.VRaptorPlugin;
import org.vraptor.webapp.WebApplication;

/**
 * Overrides all components implementing hibernate validation before executing a
 * business logic.
 * 
 * @author Guilherme Silveira
 */
public class HibernateValidatorPlugin implements VRaptorPlugin {

	private static final Logger LOG = Logger.getLogger(HibernateValidatorPlugin.class);

	final ValidatorLocator locator;

	private final ValidatorControl control;

	public HibernateValidatorPlugin() {
		locator = new ValidatorLocator();
		control = new ValidatorControl();
	}

	public void init(WebApplication application) throws LogicNotFoundException {
		LOG.info("Initializing HibernateValidatorPlugin");
		ComponentManager manager = application.getComponentManager();
		for (ComponentType component : manager.getComponents()) {
			if (control.needsToWrap(component)) {
				manager.register(control.wrapComponent(this, component));
			}
		}
	}

}
